@@ -36,7 +36,7 @@ module Cinch
     def setup
       @registration  = []
       @network       = Network.new(:unknown, :unknown)
-      @whois_updates = Hash.new {|h, k| h[k] = {}}
+      @whois_updates = {}
       @in_lists      = Set.new
     end
 
@@ -112,7 +112,15 @@ module Cinch
     # @return [void]
     # @since 2.0.0
     def send_cap_req
-      send "CAP REQ :" + ([:"away-notify", :"multi-prefix", :sasl] & @network.capabilities).join(" ")
+      caps = [:"away-notify", :"multi-prefix", :sasl] & @network.capabilities
+
+      # InspIRCd doesn't respond to empty REQs, so send an END in that
+      # case.
+      if caps.size > 0
+        send "CAP REQ :" + caps.join(" ")
+      else
+        send_cap_end
+      end
     end
 
     # @since 2.0.0
@@ -225,7 +233,7 @@ module Cinch
       msg          = Message.new(input, @bot)
       events       = [[:catchall]]
 
-      if ("001".."004").include? msg.command
+      if ["001", "002", "003", "004", "422"].include?(msg.command)
         @registration << msg.command
         if registered?
           events << [:connect]
@@ -268,7 +276,7 @@ module Cinch
 
     # @return [Boolean] true if we successfully registered yet
     def registered?
-      (("001".."004").to_a - @registration).empty?
+      (("001".."004").to_a - @registration).empty? || @registration.include?("422")
     end
 
     # Send a message to the server.
@@ -358,6 +366,11 @@ module Cinch
       end
     end
 
+    def update_whois(user, data)
+      @whois_updates[user] ||= {}
+      @whois_updates[user].merge!(data)
+    end
+
     # @since 2.0.0
     def on_away(msg, events)
       if msg.message.to_s.empty?
@@ -530,10 +543,10 @@ module Cinch
 
       if msg.message.downcase == "excess flood" && msg.user == @bot
         @bot.warn ["Looks like your bot has been kicked because of excess flood.",
-                    "If you haven't modified the throttling options manually, please file a bug report at https://github.com/cinchrb/cinch/issues and include the following information:",
-                    "- Server: #{@bot.config.server}",
-                    "- Messages per second: #{@bot.config.messages_per_second}",
-                    "- Server queue size: #{@bot.config.server_queue_size}"]
+                   "If you haven't modified the throttling options manually, please file a bug report at https://github.com/cinchrb/cinch/issues and include the following information:",
+                   "- Server: #{@bot.config.server}",
+                   "- Messages per second: #{@bot.config.messages_per_second}",
+                   "- Server queue size: #{@bot.config.server_queue_size}"]
       end
     end
 
@@ -551,7 +564,7 @@ module Cinch
 
     # @since 2.0.0
     def process_dcc_send(filename, ip, port, size, m, events)
-      if ip =~ /^\d$/
+      if ip =~ /^\d+$/
         # If ip is a single integer, assume it's a specification
         # compliant IPv4 address in network byte order. If it's any
         # other string, assume that it's a valid IPv4 or IPv6 address.
@@ -595,7 +608,7 @@ module Cinch
       away = msg.message
 
       if @whois_updates[user]
-        @whois_updates[user].merge!({:away => away})
+        update_whois(user, {:away => away})
       end
     end
 
@@ -603,36 +616,38 @@ module Cinch
     def on_307(msg, events)
       # RPL_WHOISREGNICK
       user = User(msg.params[1])
-      @whois_updates[user].merge!({:authname => user.nick})
+      update_whois(user, {:authname => user.nick})
     end
 
     def on_311(msg, events)
       # RPL_WHOISUSER
       user = User(msg.params[1])
-      @whois_updates[user].merge!({
-                                    :user => msg.params[2],
-                                    :host => msg.params[3],
-                                    :realname => msg.params[5],
-                                  })
+      update_whois(user, {
+                     :user => msg.params[2],
+                     :host => msg.params[3],
+                     :realname => msg.params[5],
+                   })
     end
 
     def on_317(msg, events)
       # RPL_WHOISIDLE
       user = User(msg.params[1])
-      @whois_updates[user].merge!({
-                                    :idle => msg.params[2].to_i,
-                                    :signed_on_at => Time.at(msg.params[3].to_i),
-                                  })
+      update_whois(user, {
+                     :idle => msg.params[2].to_i,
+                     :signed_on_at => Time.at(msg.params[3].to_i),
+                   })
     end
 
     def on_318(msg, events)
       # RPL_ENDOFWHOIS
       user = User(msg.params[1])
 
-      if @whois_updates[user].empty? && !user.attr(:unknown?, true, true)
-        user.end_of_whois(nil)
-      else
-        user.end_of_whois(@whois_updates[user])
+      if @whois_updates[user]
+        if @whois_updates[user].empty? && !user.attr(:unknown?, true, true)
+          user.end_of_whois(nil)
+        else
+          user.end_of_whois(@whois_updates[user])
+        end
         @whois_updates.delete user
       end
     end
@@ -807,10 +822,7 @@ module Cinch
 
     def on_401(msg, events)
       # ERR_NOSUCHNICK
-      user = User(msg.params[1])
-      user.sync(:unknown?, true, true)
-      msg.user.online = false
-      if @whois_updates.key?(user)
+      if user = @bot.user_list.find(msg.params[1])
         user.end_of_whois(nil, true)
         @whois_updates.delete user
       end
