@@ -294,7 +294,7 @@ module Cinch
       if @bot.irc.isupport["MONITOR"] > 0
         @bot.irc.send "MONITOR - #@name"
       else
-        @monitored_timer.stop
+        @monitored_timer.stop if @monitored_timer
       end
 
       @monitored = false
@@ -365,6 +365,10 @@ module Cinch
     # @return [void]
     def update_nick(new_nick)
       @last_nick, @name = @name, new_nick
+      # Unsync authname because some networks tie authentication to
+      # the nick, so the user might not be authenticated anymore after
+      # changing his nick
+      unsync(:authname)
       @bot.user_list.update_nick(self)
     end
 
